# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(668, 701)
        MainWindow.setWindowTitle("OpenMoldyn")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_model = QtWidgets.QWidget()
        self.tab_model.setObjectName("tab_model")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_model)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBox = QtWidgets.QGroupBox(self.tab_model)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.paramsTreeWidget = QtWidgets.QTreeWidget(self.groupBox)
        self.paramsTreeWidget.setObjectName("paramsTreeWidget")
        self.horizontalLayout.addWidget(self.paramsTreeWidget)
        self.gridLayout_2.addWidget(self.groupBox, 3, 0, 1, 8)
        self.loadSimuBtn = QtWidgets.QToolButton(self.tab_model)
        self.loadSimuBtn.setObjectName("loadSimuBtn")
        self.gridLayout_2.addWidget(self.loadSimuBtn, 1, 7, 1, 1)
        self.gotoSimuBtn = QtWidgets.QCommandLinkButton(self.tab_model)
        self.gotoSimuBtn.setEnabled(False)
        self.gotoSimuBtn.setObjectName("gotoSimuBtn")
        self.gridLayout_2.addWidget(self.gotoSimuBtn, 5, 0, 1, 8)
        self.loadModelBtn = QtWidgets.QToolButton(self.tab_model)
        self.loadModelBtn.setObjectName("loadModelBtn")
        self.gridLayout_2.addWidget(self.loadModelBtn, 1, 1, 1, 1)
        self.newModelBtn = QtWidgets.QToolButton(self.tab_model)
        self.newModelBtn.setObjectName("newModelBtn")
        self.gridLayout_2.addWidget(self.newModelBtn, 1, 0, 1, 1)
        self.saveModelBtn = QtWidgets.QToolButton(self.tab_model)
        self.saveModelBtn.setEnabled(False)
        self.saveModelBtn.setObjectName("saveModelBtn")
        self.gridLayout_2.addWidget(self.saveModelBtn, 1, 2, 1, 1)
        self.newSimuBtn = QtWidgets.QToolButton(self.tab_model)
        self.newSimuBtn.setEnabled(False)
        self.newSimuBtn.setObjectName("newSimuBtn")
        self.gridLayout_2.addWidget(self.newSimuBtn, 1, 5, 1, 1)
        self.tabWidget.addTab(self.tab_model, "")
        self.tab_simu = QtWidgets.QWidget()
        self.tab_simu.setEnabled(False)
        self.tab_simu.setObjectName("tab_simu")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tab_simu)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.progress_groupBox = QtWidgets.QGroupBox(self.tab_simu)
        self.progress_groupBox.setObjectName("progress_groupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.progress_groupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.ETA = QtWidgets.QLabel(self.progress_groupBox)
        self.ETA.setObjectName("ETA")
        self.gridLayout_4.addWidget(self.ETA, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.progress_groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 0, 2, 1, 1)
        self.simuProgressBar = QtWidgets.QProgressBar(self.progress_groupBox)
        self.simuProgressBar.setMaximum(1)
        self.simuProgressBar.setProperty("value", 0)
        self.simuProgressBar.setObjectName("simuProgressBar")
        self.gridLayout_4.addWidget(self.simuProgressBar, 1, 0, 1, 3)
        self.gridLayout_3.addWidget(self.progress_groupBox, 4, 0, 1, 2)
        self.simuBtn = QtWidgets.QCommandLinkButton(self.tab_simu)
        self.simuBtn.setObjectName("simuBtn")
        self.gridLayout_3.addWidget(self.simuBtn, 1, 1, 1, 1)
        self.gotoProcessBtn = QtWidgets.QCommandLinkButton(self.tab_simu)
        self.gotoProcessBtn.setEnabled(False)
        self.gotoProcessBtn.setObjectName("gotoProcessBtn")
        self.gridLayout_3.addWidget(self.gotoProcessBtn, 5, 0, 1, 2)
        self.simuParams = QtWidgets.QFormLayout()
        self.simuParams.setObjectName("simuParams")
        self.iterationsLabel = QtWidgets.QLabel(self.tab_simu)
        self.iterationsLabel.setObjectName("iterationsLabel")
        self.simuParams.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.iterationsLabel)
        self.iterationsSpinBox = QtWidgets.QSpinBox(self.tab_simu)
        self.iterationsSpinBox.setMaximum(210000766)
        self.iterationsSpinBox.setSingleStep(100)
        self.iterationsSpinBox.setProperty("value", 1000)
        self.iterationsSpinBox.setObjectName("iterationsSpinBox")
        self.simuParams.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.iterationsSpinBox)
        self.simulationTimeLabel = QtWidgets.QLabel(self.tab_simu)
        self.simulationTimeLabel.setObjectName("simulationTimeLabel")
        self.simuParams.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.simulationTimeLabel)
        self.simulationTimeLineEdit = QtWidgets.QLineEdit(self.tab_simu)
        self.simulationTimeLineEdit.setObjectName("simulationTimeLineEdit")
        self.simuParams.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.simulationTimeLineEdit)
        self.saveAllAtomsPositionLabel = QtWidgets.QLabel(self.tab_simu)
        self.saveAllAtomsPositionLabel.setObjectName("saveAllAtomsPositionLabel")
        self.simuParams.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.saveAllAtomsPositionLabel)
        self.saveAllAtomsPositionCheckBox = QtWidgets.QCheckBox(self.tab_simu)
        self.saveAllAtomsPositionCheckBox.setObjectName("saveAllAtomsPositionCheckBox")
        self.simuParams.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.saveAllAtomsPositionCheckBox)
        self.tryToUseGPULabel = QtWidgets.QLabel(self.tab_simu)
        self.tryToUseGPULabel.setObjectName("tryToUseGPULabel")
        self.simuParams.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.tryToUseGPULabel)
        self.tryToUseGPUCheckBox = QtWidgets.QCheckBox(self.tab_simu)
        self.tryToUseGPUCheckBox.setChecked(True)
        self.tryToUseGPUCheckBox.setTristate(False)
        self.tryToUseGPUCheckBox.setObjectName("tryToUseGPUCheckBox")
        self.simuParams.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.tryToUseGPUCheckBox)
        self.gridLayout_3.addLayout(self.simuParams, 1, 0, 1, 1)
        self.temperature_groupBox = QtWidgets.QGroupBox(self.tab_simu)
        self.temperature_groupBox.setObjectName("temperature_groupBox")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.temperature_groupBox)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.designTemperatureBtn = QtWidgets.QToolButton(self.temperature_groupBox)
        self.designTemperatureBtn.setObjectName("designTemperatureBtn")
        self.gridLayout_9.addWidget(self.designTemperatureBtn, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(522, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_9.addItem(spacerItem1, 0, 2, 1, 1)
        self.editModelBtn = QtWidgets.QToolButton(self.temperature_groupBox)
        self.editModelBtn.setEnabled(False)
        self.editModelBtn.setObjectName("editModelBtn")
        self.gridLayout_9.addWidget(self.editModelBtn, 0, 1, 1, 1)
        self.gridLayout_3.addWidget(self.temperature_groupBox, 2, 0, 1, 2)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_simu)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_6.setObjectName("gridLayout_6")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_6.addItem(spacerItem2, 1, 1, 1, 1)
        self.RTViewBtn = QtWidgets.QToolButton(self.groupBox_3)
        self.RTViewBtn.setObjectName("RTViewBtn")
        self.gridLayout_6.addWidget(self.RTViewBtn, 1, 2, 1, 1)
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.currentIterationLabel = QtWidgets.QLabel(self.groupBox_3)
        self.currentIterationLabel.setObjectName("currentIterationLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.currentIterationLabel)
        self.currentIteration = QtWidgets.QLabel(self.groupBox_3)
        self.currentIteration.setObjectName("currentIteration")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.currentIteration)
        self.label_4 = QtWidgets.QLabel(self.groupBox_3)
        self.label_4.setObjectName("label_4")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.currentTime = QtWidgets.QLabel(self.groupBox_3)
        self.currentTime.setObjectName("currentTime")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.currentTime)
        self.gridLayout_6.addLayout(self.formLayout_2, 1, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_3, 3, 0, 1, 2)
        self.tabWidget.addTab(self.tab_simu, "")
        self.tab_processing = QtWidgets.QWidget()
        self.tab_processing.setEnabled(False)
        self.tab_processing.setObjectName("tab_processing")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_processing)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.groupBox1d = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox1d.setObjectName("groupBox1d")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox1d)
        self.gridLayout.setObjectName("gridLayout")
        self.lineListW = QtWidgets.QListWidget(self.groupBox1d)
        self.lineListW.setObjectName("lineListW")
        self.gridLayout.addWidget(self.lineListW, 0, 0, 2, 1)
        self.formLayout_3 = QtWidgets.QFormLayout()
        self.formLayout_3.setObjectName("formLayout_3")
        self.asAFunctionOfLabel = QtWidgets.QLabel(self.groupBox1d)
        self.asAFunctionOfLabel.setObjectName("asAFunctionOfLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.asAFunctionOfLabel)
        self.lineComboW = QtWidgets.QComboBox(self.groupBox1d)
        self.lineComboW.setObjectName("lineComboW")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineComboW)
        self.plotB = QtWidgets.QPushButton(self.groupBox1d)
        self.plotB.setObjectName("plotB")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.plotB)
        self.gridLayout.addLayout(self.formLayout_3, 0, 1, 1, 1)
        self.gridLayout_5.addWidget(self.groupBox1d, 2, 0, 1, 1)
        self.groupBox_1doptions = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox_1doptions.setObjectName("groupBox_1doptions")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.groupBox_1doptions)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.list1DOptions = QtWidgets.QListWidget(self.groupBox_1doptions)
        self.list1DOptions.setObjectName("list1DOptions")
        self.horizontalLayout_4.addWidget(self.list1DOptions)
        self.gridLayout_5.addWidget(self.groupBox_1doptions, 2, 1, 1, 1)
        self.groupBoxMovie = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBoxMovie.setObjectName("groupBoxMovie")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBoxMovie)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.makeMovieBtn = QtWidgets.QToolButton(self.groupBoxMovie)
        self.makeMovieBtn.setObjectName("makeMovieBtn")
        self.gridLayout_8.addWidget(self.makeMovieBtn, 0, 2, 1, 1)
        self.formLayout_4 = QtWidgets.QFormLayout()
        self.formLayout_4.setObjectName("formLayout_4")
        self.fPSLabel = QtWidgets.QLabel(self.groupBoxMovie)
        self.fPSLabel.setObjectName("fPSLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.fPSLabel)
        self.FPSSpinBox = QtWidgets.QSpinBox(self.groupBoxMovie)
        self.FPSSpinBox.setProperty("value", 24)
        self.FPSSpinBox.setObjectName("FPSSpinBox")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.FPSSpinBox)
        self.stepsByFrameLabel = QtWidgets.QLabel(self.groupBoxMovie)
        self.stepsByFrameLabel.setObjectName("stepsByFrameLabel")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.stepsByFrameLabel)
        self.stepsByFrameSpinBox = QtWidgets.QSpinBox(self.groupBoxMovie)
        self.stepsByFrameSpinBox.setProperty("value", 10)
        self.stepsByFrameSpinBox.setObjectName("stepsByFrameSpinBox")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.stepsByFrameSpinBox)
        self.gridLayout_8.addLayout(self.formLayout_4, 0, 0, 1, 1)
        self.movieProgressBar = QtWidgets.QProgressBar(self.groupBoxMovie)
        self.movieProgressBar.setProperty("value", 0)
        self.movieProgressBar.setObjectName("movieProgressBar")
        self.gridLayout_8.addWidget(self.movieProgressBar, 0, 1, 1, 1)
        self.gridLayout_5.addWidget(self.groupBoxMovie, 5, 0, 1, 2)
        self.groupBox1d_stat = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox1d_stat.setObjectName("groupBox1d_stat")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.groupBox1d_stat)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.PDFButton = QtWidgets.QToolButton(self.groupBox1d_stat)
        self.PDFButton.setObjectName("PDFButton")
        self.horizontalLayout_5.addWidget(self.PDFButton)
        self.label = QtWidgets.QLabel(self.groupBox1d_stat)
        self.label.setObjectName("label")
        self.horizontalLayout_5.addWidget(self.label)
        self.PDFNSpinBox = QtWidgets.QSpinBox(self.groupBox1d_stat)
        self.PDFNSpinBox.setMinimum(1)
        self.PDFNSpinBox.setMaximum(100000)
        self.PDFNSpinBox.setSingleStep(100)
        self.PDFNSpinBox.setProperty("value", 1000)
        self.PDFNSpinBox.setObjectName("PDFNSpinBox")
        self.horizontalLayout_5.addWidget(self.PDFNSpinBox)
        self.label_2 = QtWidgets.QLabel(self.groupBox1d_stat)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_5.addWidget(self.label_2)
        self.PDFDistSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox1d_stat)
        self.PDFDistSpinBox.setSingleStep(0.1)
        self.PDFDistSpinBox.setProperty("value", 1.5)
        self.PDFDistSpinBox.setObjectName("PDFDistSpinBox")
        self.horizontalLayout_5.addWidget(self.PDFDistSpinBox)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.gridLayout_5.addWidget(self.groupBox1d_stat, 1, 0, 1, 2)
        self.groupBox_data_mgt = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox_data_mgt.setObjectName("groupBox_data_mgt")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_data_mgt)
        self.gridLayout_7.setObjectName("gridLayout_7")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem4, 0, 3, 1, 1)
        self.saveSimuBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.saveSimuBtn.setObjectName("saveSimuBtn")
        self.gridLayout_7.addWidget(self.saveSimuBtn, 0, 2, 1, 1)
        self.saveRModelBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.saveRModelBtn.setObjectName("saveRModelBtn")
        self.gridLayout_7.addWidget(self.saveRModelBtn, 0, 0, 1, 1)
        self.reuseModelBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.reuseModelBtn.setObjectName("reuseModelBtn")
        self.gridLayout_7.addWidget(self.reuseModelBtn, 0, 1, 1, 1)
        self.exportBtn = QtWidgets.QToolButton(self.groupBox_data_mgt)
        self.exportBtn.setObjectName("exportBtn")
        self.gridLayout_7.addWidget(self.exportBtn, 0, 4, 1, 1)
        self.gridLayout_5.addWidget(self.groupBox_data_mgt, 0, 0, 1, 2)
        self.groupBox2d = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox2d.setObjectName("groupBox2d")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox2d)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.surfListW = QtWidgets.QListWidget(self.groupBox2d)
        self.surfListW.setObjectName("surfListW")
        self.gridLayout_10.addWidget(self.surfListW, 0, 0, 1, 1)
        self.drawSurfButton = QtWidgets.QToolButton(self.groupBox2d)
        self.drawSurfButton.setObjectName("drawSurfButton")
        self.gridLayout_10.addWidget(self.drawSurfButton, 0, 2, 1, 1)
        self.gridLayout_5.addWidget(self.groupBox2d, 3, 0, 1, 2)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab_processing)
        self.groupBox_2.setObjectName("groupBox_2")
        self.formLayout = QtWidgets.QFormLayout(self.groupBox_2)
        self.formLayout.setObjectName("formLayout")
        self.deformationDistLabel = QtWidgets.QLabel(self.groupBox_2)
        self.deformationDistLabel.setObjectName("deformationDistLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.deformationDistLabel)
        self.deformationDistSpinBox = QtWidgets.QDoubleSpinBox(self.groupBox_2)
        self.deformationDistSpinBox.setSingleStep(0.1)
        self.deformationDistSpinBox.setProperty("value", 2.0)
        self.deformationDistSpinBox.setObjectName("deformationDistSpinBox")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.deformationDistSpinBox)
        self.gridLayout_5.addWidget(self.groupBox_2, 4, 0, 1, 2)
        self.tabWidget.addTab(self.tab_processing, "")
        self.horizontalLayout_2.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 668, 21))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionNew = QtWidgets.QAction(MainWindow)
        self.actionNew.setObjectName("actionNew")
        self.actionOpen_model = QtWidgets.QAction(MainWindow)
        self.actionOpen_model.setObjectName("actionOpen_model")
        self.actionNew_simulation = QtWidgets.QAction(MainWindow)
        self.actionNew_simulation.setObjectName("actionNew_simulation")
        self.iterationsLabel.setBuddy(self.iterationsSpinBox)
        self.simulationTimeLabel.setBuddy(self.simulationTimeLineEdit)
        self.saveAllAtomsPositionLabel.setBuddy(self.saveAllAtomsPositionCheckBox)
        self.tryToUseGPULabel.setBuddy(self.tryToUseGPUCheckBox)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.newModelBtn, self.loadModelBtn)
        MainWindow.setTabOrder(self.loadModelBtn, self.saveModelBtn)
        MainWindow.setTabOrder(self.saveModelBtn, self.loadSimuBtn)
        MainWindow.setTabOrder(self.loadSimuBtn, self.paramsTreeWidget)
        MainWindow.setTabOrder(self.paramsTreeWidget, self.gotoSimuBtn)
        MainWindow.setTabOrder(self.gotoSimuBtn, self.iterationsSpinBox)
        MainWindow.setTabOrder(self.iterationsSpinBox, self.simulationTimeLineEdit)
        MainWindow.setTabOrder(self.simulationTimeLineEdit, self.saveAllAtomsPositionCheckBox)
        MainWindow.setTabOrder(self.saveAllAtomsPositionCheckBox, self.tryToUseGPUCheckBox)
        MainWindow.setTabOrder(self.tryToUseGPUCheckBox, self.simuBtn)
        MainWindow.setTabOrder(self.simuBtn, self.RTViewBtn)
        MainWindow.setTabOrder(self.RTViewBtn, self.gotoProcessBtn)
        MainWindow.setTabOrder(self.gotoProcessBtn, self.saveRModelBtn)
        MainWindow.setTabOrder(self.saveRModelBtn, self.reuseModelBtn)
        MainWindow.setTabOrder(self.reuseModelBtn, self.saveSimuBtn)
        MainWindow.setTabOrder(self.saveSimuBtn, self.PDFButton)
        MainWindow.setTabOrder(self.PDFButton, self.PDFNSpinBox)
        MainWindow.setTabOrder(self.PDFNSpinBox, self.PDFDistSpinBox)
        MainWindow.setTabOrder(self.PDFDistSpinBox, self.lineListW)
        MainWindow.setTabOrder(self.lineListW, self.lineComboW)
        MainWindow.setTabOrder(self.lineComboW, self.plotB)
        MainWindow.setTabOrder(self.plotB, self.list1DOptions)
        MainWindow.setTabOrder(self.list1DOptions, self.surfListW)
        MainWindow.setTabOrder(self.surfListW, self.drawSurfButton)
        MainWindow.setTabOrder(self.drawSurfButton, self.FPSSpinBox)
        MainWindow.setTabOrder(self.FPSSpinBox, self.stepsByFrameSpinBox)
        MainWindow.setTabOrder(self.stepsByFrameSpinBox, self.makeMovieBtn)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        self.groupBox.setTitle(_translate("MainWindow", "Current model"))
        self.paramsTreeWidget.headerItem().setText(0, _translate("MainWindow", "Property"))
        self.paramsTreeWidget.headerItem().setText(1, _translate("MainWindow", "Value"))
        self.paramsTreeWidget.headerItem().setText(2, _translate("MainWindow", "Unit"))
        self.loadSimuBtn.setText(_translate("MainWindow", "Load simulation history"))
        self.gotoSimuBtn.setText(_translate("MainWindow", "Simulation"))
        self.loadModelBtn.setText(_translate("MainWindow", "Load"))
        self.newModelBtn.setText(_translate("MainWindow", "New"))
        self.saveModelBtn.setText(_translate("MainWindow", "Save"))
        self.newSimuBtn.setText(_translate("MainWindow", "New simulation from this model"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_model), _translate("MainWindow", "Model"))
        self.progress_groupBox.setTitle(_translate("MainWindow", "Progress"))
        self.ETA.setText(_translate("MainWindow", "0"))
        self.label_3.setText(_translate("MainWindow", "ETA : "))
        self.simuProgressBar.setFormat(_translate("MainWindow", "%v/%m"))
        self.simuBtn.setText(_translate("MainWindow", "Launch simulation"))
        self.gotoProcessBtn.setText(_translate("MainWindow", "Process data"))
        self.iterationsLabel.setText(_translate("MainWindow", "Iterations"))
        self.simulationTimeLabel.setText(_translate("MainWindow", "Simulation time (s)"))
        self.saveAllAtomsPositionLabel.setText(_translate("MainWindow", "Save all atoms position"))
        self.tryToUseGPULabel.setText(_translate("MainWindow", "Try to use GPU"))
        self.temperature_groupBox.setTitle(_translate("MainWindow", "Model tweaking"))
        self.designTemperatureBtn.setText(_translate("MainWindow", "Design temperature evolution"))
        self.editModelBtn.setText(_translate("MainWindow", "Edit"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Current state"))
        self.RTViewBtn.setText(_translate("MainWindow", "View current state"))
        self.currentIterationLabel.setText(_translate("MainWindow", "Current iteration : "))
        self.currentIteration.setText(_translate("MainWindow", "0"))
        self.label_4.setText(_translate("MainWindow", "Current time (s) : "))
        self.currentTime.setText(_translate("MainWindow", "0"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_simu), _translate("MainWindow", "Simulation"))
        self.groupBox1d.setTitle(_translate("MainWindow", "1D graphics"))
        self.asAFunctionOfLabel.setText(_translate("MainWindow", "as a function of"))
        self.plotB.setText(_translate("MainWindow", "Plot"))
        self.groupBox_1doptions.setTitle(_translate("MainWindow", "1D options"))
        self.groupBoxMovie.setTitle(_translate("MainWindow", "Movie"))
        self.makeMovieBtn.setText(_translate("MainWindow", "Make movie"))
        self.fPSLabel.setText(_translate("MainWindow", "FPS"))
        self.stepsByFrameLabel.setText(_translate("MainWindow", "Steps by frame"))
        self.groupBox1d_stat.setTitle(_translate("MainWindow", "Static 1D graphics"))
        self.PDFButton.setText(_translate("MainWindow", "Pair Distribution Function"))
        self.label.setText(_translate("MainWindow", "Number of atoms :"))
        self.label_2.setText(_translate("MainWindow", "Maximum distance (times r_cut) :"))
        self.groupBox_data_mgt.setTitle(_translate("MainWindow", "Data management"))
        self.saveSimuBtn.setText(_translate("MainWindow", "Save simulation history"))
        self.saveRModelBtn.setText(_translate("MainWindow", "Save resulting model"))
        self.reuseModelBtn.setText(_translate("MainWindow", "New simulation with resulting model"))
        self.exportBtn.setText(_translate("MainWindow", "Export to CSV"))
        self.groupBox2d.setTitle(_translate("MainWindow", "2D graphics"))
        self.drawSurfButton.setText(_translate("MainWindow", "Draw"))
        self.groupBox_2.setTitle(_translate("MainWindow", "2D parameters"))
        self.deformationDistLabel.setText(_translate("MainWindow", "Maximum distance for calculation (times r_cut)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_processing), _translate("MainWindow", "Processing"))
        self.actionNew.setText(_translate("MainWindow", "New model"))
        self.actionOpen_model.setText(_translate("MainWindow", "Open model"))
        self.actionNew_simulation.setText(_translate("MainWindow", "New simulation"))




# -*-encoding: utf-8 -*-
import io
import os

from PIL import Image
from imageio_ffmpeg import write_frames

from moldyn.processing.data_proc import compute_strain
from . import data_proc as dp
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
from functools import wraps
import matplotlib
try:
    matplotlib.use('Qt5Agg')
except:
    pass
import numpy as np


def _plot_base(*, show=False, axis='', grid=False, figure=None):
    """
    A wrapper designed to handle basic matplotlib configuration.

    Parameters
    ----------
    show : bool
        If True, will show the plot.
    axis : str
        Configure the plot axis ('equal, ...).
    grid : bool
        Activate the grid on True.
    figure : int
        Specify to which figure to plot. If not specified (ie. None),
        will use the last used figure.

    Returns
    -------
    The wrapped function taking the same arguments as the original.

    """
    def plot_dec(func):
        @wraps(func)
        def wrap(*args, **kwargs):
            keys = kwargs.keys()
            if 'figure' in keys:
                if isinstance(kwargs['figure'], plt.Figure):
                    num = kwargs['figure'].number
                else:
                    num = kwargs['figure']
                plt.figure(num)
                del kwargs['figure']
            elif figure is not None:
                plt.figure(figure)

            if 'grid' in keys and kwargs['grid']:
                plt.grid()
                del kwargs['grid']
            elif grid:
                plt.grid()

            func(*args, **kwargs)

            plt.axis(axis)
            if show:
                plt.show()
        return wrap
    return plot_dec



@_plot_base(axis='scaled', grid=False)
def plot_density(model, levels=None, refinement=0):
    """
    Compute and plot the contours of the local density of model.

    Parameters
    ----------
    model : Model
        The model to plot
    levels : int or array-like
        the number of levels or a sorted array-like object of levels
    refinement : int
        the level of refinement
    Returns
    -------

    """
    tri, density = dp.density(model, refinement)
    fig = plt.gcf()
    if type(levels) == int:
        levels = np.linspace(min(density), max(density), levels)
    CS = plt.tricontour(tri, density, levels=levels)
    cbar = fig.colorbar()
    cbar.add_lines(CS)
    cbar.ax.set_ylabel('local density')


@_plot_base(axis='scaled', grid=False)
def plot_densityf(model, levels=None, refinement=0):
    """
    Compute and plot the filled contours of the local density of model.

    Parameters
    ----------
    model : Model
        The model to plot
    levels : int or array-like
        the number of levels or a sorted array-like object of levels
    refinement : int
        the level of refinement
    Returns
    -------

    """
    tri, density = dp.density(model, refinement)
    fig = plt.gcf()
    cmap = plt.get_cmap("viridis")
    if type(levels) == int:
        levels = np.linspace(min(density), max(density), levels)
    CS = plt.tricontourf(tri, density, levels=levels, cmap=cmap)
    cbar = fig.colorbar(CS)
    cbar.ax.set_ylabel('local density')

@_plot_base(axis='scaled', grid=False)
def plot_particles(model):
    """
    Plot the position stored in model.

    Parameters
    ----------
    model : Model
        The model to plot
    """
    line1, = plt.plot(*model.pos[:model.n_a, :].T, 'ro', markersize=0.5)
    line2, = plt.plot(*model.pos[model.n_a:, :].T, 'bo', markersize=0.5)

@_plot_base(axis='scaled', grid=False)
def plot_density_surf(model,refinement=0):
    """
    Compute and plot the 3D surface of the local density of model.

    Parameters
    ----------
    model : Model
        The model to plot.
    refinement : int
        the level of refinement
    Returns
    -------

    """
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    tri, density = dp.density(model, refinement)
    ax.plot_trisurf(tri, density)



def make_movie(simulation, dynstate, name: str, pfilm=5, fps=24, callback=None):
    """
    Makes a .mp4 movie of simulation from the history saved in dynstate.

    Parameters
    ----------
    simulation : Simulation
        The simulation object
    dynstate : DynState
        The dynState object containing the position history file.
    name : str
        The path (and name) of the file to be writen.
    pfilm : int
        To make the movie, takes every pfilm position.
    fps : int
        The number of frame per seconds of the film.
    callback : function
        An optional callback function that is called every time a
        frame is made and is passed the current iteration number.

    Returns
    -------

    """
    # ouverture pour la lecture
    npas = simulation.current_iter
    YlimB = simulation.model.y_lim_inf
    YlimH = simulation.model.y_lim_sup
    XlimG = simulation.model.x_lim_inf
    XlimD = simulation.model.x_lim_sup
    if not name.endswith(".mp4"):
        name += ".mp4"
    with dynstate.open(dynstate.POS_H, 'r') as fix:
        # liste de k ou tracer le graph
        klist = set(range(0, npas, pfilm))
        # boucle pour creer le film
        figure_size = (1920, 1088)
        try:
            os.remove(name)
        except FileNotFoundError:
            pass
        gen = write_frames(name, figure_size, fps=fps, quality=9)
        gen.send(None)
        fig = plt.figure(figsize=(figure_size[0] / (72 * 2), figure_size[1] / (72 * 2)))
        plt.clf()
        # definition du domaine de dessin
        plt.ioff()  # pour ne pas afficher les graphs)
        plt.axis('scaled')
        plt.ylim(YlimB, YlimH)
        plt.xlim(XlimG, XlimD)
        pos = fix.load()
        line1, = plt.plot(*pos[:simulation.model.n_a,:].T, 'ro', markersize=0.5)
        line2, = plt.plot(*pos[simulation.model.n_a:,:].T, 'bo', markersize=0.5)
        plt.xlabel("0")
        temp = io.BytesIO()
        for k in range(1, npas):
            # dessin a chaque pas (ne s'ouvre pas: est sauvegarde de maniere incrementale)
            if k in klist:
                temp.seek(0)
                plt.xlabel("Iteration : {}".format(k))
                plt.title(f"T = {simulation.state_fct['T'][k]:.2f} K")
                line1.set_data(*pos[:simulation.model.n_a,:].T)
                line2.set_data(*pos[simulation.model.n_a:,:].T)
                fig.savefig(temp, format='raw', dpi=72 * 2)  # sauvegarde incrementale
                temp.seek(0)
                if callback: callback(k)
                gen.send(Image.frombytes('RGBA', figure_size, temp.read()).convert('RGB').tobytes())
            pos = fix.load() # on charge a chaque pas de temps
        gen.close()
        plt.close(fig)


@_plot_base(axis='scaled', grid=False)
def deformation_volume(model0, model1, rcut, levels=50):
    eps = compute_strain(model0, model1, rcut)
    fig = plt.gcf()
    cmap = plt.get_cmap("plasma")
    dV = eps.trace(axis1=1, axis2=2)
    if type(levels) == int:
        levels = np.linspace(min(dV), max(dV)+1e-30, levels)
    CS = plt.tricontourf(*model1.pos.T, dV, levels=levels, cmap=cmap)
    cbar = fig.colorbar(CS)
    cbar.ax.set_ylabel('local compression')

@_plot_base(axis='scaled', grid=False)
def deformation_dev(model0, model1, rcut, levels=50):
    eps = compute_strain(model0, model1, rcut)
    fig = plt.gcf()
    cmap = plt.get_cmap("plasma")
    dV = eps[:,0,1]
    if type(levels) == int:
        levels = np.linspace(min(dV), max(dV)+1e-30, levels)
    CS = plt.tricontourf(*model1.pos.T, dV, levels=levels, cmap=cmap)
    cbar = fig.colorbar(CS)
    cbar.ax.set_ylabel('local shear')



# -*-encoding: utf-8 -*-

from ..utils import gl_util
import os
import moderngl
import numpy as np

if not gl_util.testGL():
    os.environ["MESA_GL_VERSION_OVERRIDE"] = "4.3"
    os.environ["MESA_GLSL_VERSION_OVERRIDE"] = "430"


class ForcesComputeGPU:
    """
    Compute module. Runs on GPU.


    Parameters
    ----------
    consts : dict
        Dictionary containing constants used for calculations.

    Attributes
    ----------
    npart : int
        number of atoms
    consts : dict
        Dictionary containing constants used for calculations, and some parameters to run the compute shader.

    """

    def __init__(self, consts, compute_npart=None):

        self.npart = consts["NPART"]
        self.compute_npart = compute_npart or consts["NPART"]

        max_layout_size = 256  # Probablement optimal (en tout cas d'après essais et guides de bonnes pratiques)
        self.groups_number = int(np.ceil(self.compute_npart / max_layout_size))
        self.layout_size = int(np.ceil(self.compute_npart / self.groups_number))

        consts["LAYOUT_SIZE"] = self.layout_size

        self.compute_npart = min(self.compute_npart, self.npart)
        self.compute_offset = 0

        self.context = moderngl.create_standalone_context(require=430)
        self.compute_shader = self.context.compute_shader(gl_util.source(os.path.dirname(__file__)+'/templates/moldyn.glsl', consts))


        self.consts = consts

        # Buffer de positions 1
        self._BUFFER_P = self.context.buffer(reserve=2 * 4 * self.npart)
        self._BUFFER_P.bind_to_storage_buffer(0)

        # Buffer de forces
        self._BUFFER_F = self.context.buffer(reserve=2 * 4 * self.npart)
        self._BUFFER_F.bind_to_storage_buffer(1)

        # Buffer d'énergies potentielles
        self._BUFFER_E = self.context.buffer(reserve=4 * self.npart)
        self._BUFFER_E.bind_to_storage_buffer(2)

        # Buffer de compteurs de liaisons
        self._BUFFER_COUNT = self.context.buffer(reserve=4 * self.npart)
        self._BUFFER_COUNT.bind_to_storage_buffer(3)

        # Buffer de paramètres, inutilisé pour l'instant
        self._BUFFER_PARAMS = self.context.buffer(reserve=4 * 5)
        self._BUFFER_PARAMS.bind_to_storage_buffer(4)

        self.array_shape = (self.npart, 2)

    def set_pos(self, pos):
        """
        Set position array and launches forces computation.

        Parameters
        ----------
        pos : np.ndarray
            Array of positions.

        Returns
        -------

        """
        self._BUFFER_P.write(pos.astype('f4').tobytes())
        self.compute_shader.run(group_x=self.groups_number)

    def get_F(self):
        """

        Returns
        -------
        np.ndarray
            Computed inter-atomic forces.
        """
        return np.frombuffer(self._BUFFER_F.read(), dtype=np.float32).reshape(self.array_shape)

    def get_PE(self):
        """

        Returns
        -------
        np.ndarray
            Computed potential energy.
        """
        return np.frombuffer(self._BUFFER_E.read(), dtype=np.float32)

    def get_COUNT(self):
        """

        Returns
        -------
        np.ndarray
            Average near atoms.
        """
        return np.frombuffer(self._BUFFER_COUNT.read(), dtype=np.float32)


"""Tests for syncing directories using rsync

"""

import datreant as dtr
import pytest
import os
import py.path


def test_sync_local(tmpdir):
    '''Test that syncronization works locally'''
    with tmpdir.as_cwd():
        sequoia = dtr.Tree("sequoia").makedirs()
        sequoia2 = dtr.Tree("sequoia2").makedirs()
        py.path.local('sequoia/hello.txt').write('hello')

        assert os.path.exists('sequoia/hello.txt')

        sequoia.sync(sequoia2)
        assert os.path.exists('sequoia2/hello.txt')


def _create_tree(name, files=[]):
    tree = dtr.Tree(name).makedirs()
    for file_ in files:
        py.path.local(os.path.join(tree.abspath, file_)
                      ).write('hello', ensure=True)

    return tree


def test_overwrite(tmpdir):
    with tmpdir.as_cwd():
        sequoia = _create_tree('sequoia', ['hello.txt', 'data/hello.txt',
                                           'data/world.dat', 'world.dat'])
        sequoia2 = dtr.Tree("sequoia2").makedirs()

        # Upload contents
        sequoia.sync(sequoia2, mode='upload')

        # Change contents
        open(sequoia2['hello.txt'].abspath, 'w').write('newcontent')

        # Upload contents again
        sequoia.sync(sequoia2, mode='upload')

        # Verify that the hello.txt is not overwritten
        assert sequoia2['hello.txt'].read() == 'newcontent'


def test_excludes(tmpdir):
    with tmpdir.as_cwd():
        sequoia = _create_tree('sequoia', ['hello.txt', 'data/hello.txt',
                                           'data/world.dat', 'world.dat'])
        sequoia2 = dtr.Tree("sequoia2").makedirs()
        sequoia3 = dtr.Tree("sequoia3").makedirs()

        sequoia.sync(sequoia2, exclude='*.txt')

        assert os.path.exists('sequoia2/world.dat')
        assert os.path.exists('sequoia2/data/world.dat')

        assert not os.path.exists('sequoia2/hello.txt')
        assert not os.path.exists('sequoia2/data/hello.txt')

        sequoia.sync(sequoia3, exclude=['*.txt', '*.dat'])
        assert not os.path.exists('sequoia3/hello.txt')
        assert not os.path.exists('sequoia3/world.dat')

        assert os.path.exists('sequoia3/data/')


def test_includes(tmpdir):
    with tmpdir.as_cwd():
        sequoia = _create_tree('sequoia', ['hello.txt', 'data/hello.txt',
                                           'data/world.dat', 'world.dat'])
        sequoia2 = dtr.Tree("sequoia2").makedirs()
        sequoia3 = dtr.Tree("sequoia3").makedirs()
        sequoia4 = dtr.Tree("sequoia4").makedirs()

        # Only txt
        sequoia.sync(sequoia2, include='*.txt')

        assert os.path.exists('sequoia2/data/hello.txt')
        assert os.path.exists('sequoia2/hello.txt')

        assert not os.path.exists('sequoia2/world.dat')
        assert not os.path.exists('sequoia2/data/world.dat')

        # Only txt and dat
        sequoia.sync(sequoia3, include=['*.txt', '*.dat'])

        assert os.path.exists('sequoia3/data/hello.txt')
        assert os.path.exists('sequoia3/hello.txt')

        assert os.path.exists('sequoia3/world.dat')
        assert os.path.exists('sequoia3/data/world.dat')

        # We can also test include/excludes at the same time
        sequoia.sync(sequoia4, exclude='*.txt', include=['data/*'])

        assert os.path.exists('sequoia4/data/world.dat')
        assert os.path.exists('sequoia4/data/hello.txt')
        assert not os.path.exists('sequoia4/hello.txt')
        assert not os.path.exists('sequoia4/world.dat')


# def test_sync_remote(tmpdir):
#     remote_path = 'user@host:/tmp/sequoia'
#     with tmpdir.as_cwd():
#         sequoia = dtr.Tree("sequoia").makedirs()
#         py.path.local('sequoia/hello.txt').write('hello')
#
#         sequoia2 = dtr.Tree("sequoia2").makedirs()
#
#         # Upload
#         sequoia.sync(remote_path)
#
#         # Download
#         sequoia2.sync(remote_path, mode='download')
#
#         assert os.path.exists('sequoia2/hello.txt')


# def test_rsync_tree(tmpdir):
#
#     # We want to sync this tree to another directory
#     with tmpdir.as_cwd():
#         # Let's assume we have an existing tree
#         dtr.Tree("sequoia").makedirs()
#
#         # We create a new tree
#         t = dtr.Tree('sequoia2')
#
#         # Options
#         t.sync('sequoia',
#                mode='download',
#                compress=True,
#                backup=True,
#                dry=True,
#                include=["*.txt"],
#                exclude=[])
#
#         # Upload content to sequoia
#         t.sync('sequoia', mode='upload')
#
#         # Download content from sequoia
#         t.sync('sequoia', mode='download')
#
#         # Remote sync
#         t.sync('sequoia2', host='127.0.0.1', user='xxx', password='xxx')
#         t.sync('sequoia2', host='127.0.0.1', user='xxx', key='./cloud.pem')



"""Tests for Trees and Leaves.

"""

import pytest
import os
import py

import datreant as dtr
from datreant import Veg, Leaf, Tree, Treant


class TestVeg(object):
    """Common element tests of Trees and Leaves"""
    cls = Veg
    name = 'veggie'

    @pytest.fixture
    def veg(self, tmpdir):
        with tmpdir.as_cwd():
            v = Veg(self.name)
            yield v

    def test_str(self, veg):
        assert str(veg) == os.path.join(os.getcwd(), self.name)

    def test_hash(self, veg):
        # hases are based only on abspath
        vset = {veg}
        assert veg in vset
        v2 = self.cls(self.name)
        assert v2 in vset

    def test_exists(self, veg):
        assert not veg.exists

    def test_abspath(self, veg):
        assert veg.abspath == os.path.join(os.getcwd(), self.name)

    def test_relpath(self, veg):
        assert veg.relpath == self.name

    def test_parent(self, veg):
        p = veg.parent
        assert isinstance(p, Tree)
        assert p == Tree(os.path.split(self.name)[0])

    def test_name(self, veg):
        assert veg.name == os.path.split(self.name)[1]


class TestTree(TestVeg):
    """Test generic Treant features"""
    cls = Tree
    name = 'testtreant'

    @pytest.fixture
    def tree(self, tmpdir):
        with tmpdir.as_cwd():
            t = Tree(self.name)
            yield t
    veg = tree

    def test_abspath(self, veg):
        assert veg.abspath == os.path.join(os.getcwd(), self.name) + os.sep

    def test_relpath(self, veg):
        assert veg.relpath == self.name + os.sep

    class TestTreeInit(object):
        """Test tree init.

        Test that tree works for:
            1. nonexistent directory
            2. existing directory

        Test that exception raised for:
            1. tree initialized with existing file

        """
        def test_nonexistant(self, tmpdir):
            with tmpdir.as_cwd():
                # test nonexistent directory
                t = Tree('bark')
                assert isinstance(t, Tree)
                assert not t.exists

        def test_existing(self, tmpdir):
            with tmpdir.as_cwd():
                os.makedirs('bark/lark')
                t = Tree('bark/lark/')
                assert isinstance(t, Tree)
                assert t.exists

        def test_file_ValueError(self, tmpdir):
            with tmpdir.as_cwd():
                os.mkdir('bark')
                with open(os.path.join('bark', 'mark.txt'), 'w') as f:
                    f.write('hello\nthis is a cool file\n')
                with pytest.raises(ValueError):
                    t = Tree('bark/mark.txt')

    def test_exists(self, tree):
        tree.make()

        assert os.path.exists(tree.abspath)
        assert tree.exists is True

    def test_not_exists(self, tree):
        assert not tree.exists

    @pytest.fixture
    def contains_Tree(self, tmpdir):
        # Contains various combinations of files and directories
        # that do and do not exist. Structure:
        # container
        # + dir1
        #   + file2
        # + dir3  # doesn't exist
        #   + file4  # doesn't exist
        with tmpdir.as_cwd():
            os.mkdir('container')
            os.mkdir(os.path.join('container', 'dir1'))
            with open(os.path.join('container', 'dir1', 'file2'), 'w') as f:
                f.write('some data here\n')
            tree = Tree('container')
            yield tree

    @pytest.mark.parametrize('path,exp', (
        ('dir1/', True),
        ('dir1/file2', True),
        ('dir3/', False),
        ('dir3/file4', False),
    ))
    def test_exists(self, contains_Tree, path, exp):
        assert contains_Tree[path].exists == exp

    @pytest.mark.parametrize('path,exp', (
        (os.path.join(name, 'thing1'), True),
        (os.path.join(name, 'thing2', 'thing3'), True),
        (os.path.join('other', 'thing1'), False),
    ))
    # loop over possible input types
    @pytest.mark.parametrize('inptype', (Leaf, Tree, str))
    def test_contains(self, tree, inptype, path, exp):
        thing = inptype(path)  # convert to desired type
        assert (thing in tree) == exp

    def test_contains_TypeError(self, tree):
        with pytest.raises(TypeError):
            24.0 in tree

    def test_loc(self, tree):
        l = tree.loc
        assert isinstance(l, dtr.trees._Loc)

    @pytest.fixture(params=[
        lambda x: x,  # ie just pass on the tree
        lambda x: getattr(x, 'loc')  # pass on the .loc of it
    ])
    def tree_getitem(self, tree, request):
        # for testing getitem, yields either the Tree or the '.loc' of it
        # these should be equivalent
        yield request.param(tree)

    def test_getitem_subtree(self, tree, tree_getitem):
        subt = tree_getitem['ground/control/to/major/treebeard/']
        assert isinstance(subt, Tree)
        assert not subt.exists
        assert subt.path
        assert subt in tree

    def test_getitem_leaf(self, tree, tree_getitem):
        leaf = tree_getitem['this/is/a/file']
        assert isinstance(leaf, Leaf)
        assert leaf in tree

    def test_getitem_many_leaves(self, tree_getitem):
        v = tree_getitem[['a/file', 'a/tree/']]
        assert len(v) == 2
        assert len(v.memberleaves) == 1
        assert len(v.membertrees) == 1

    def test_getitem_ValueError(self, tree_getitem):
        with pytest.raises(ValueError):
            tree_getitem['lolcats', 'a/not/file']

    def test_getitem_returntypes(self, tree):
        tree['ground/hogs/on/mars/'].make()
        tree['the/file/of your/life'].make()
        v = tree[['ground/hogs/on/mars', 'the/file/of your/life']]

        assert isinstance(v[0], Tree)
        assert isinstance(v[1], Leaf)

    def test_leaves(self, tree):
        with pytest.raises(OSError):
            tree.leaves()

        # actually make the directory now
        tree.makedirs()

        tree['.hide/me'].make()
        tree['.hide/here/'].make()

        assert len(tree.leaves()) == 0

        tree['thing1'].make()
        tree['thing2'].make()
        tree['thing3'].make()

        assert len(tree.leaves()) == 3

        tree['larry/'].make()
        tree['curly/'].make()

        assert len(tree.leaves()) == 3

    def test_trees(self, tree):
        with pytest.raises(OSError):
            tree.trees()

        # actually make the directory now
        tree.makedirs()

        assert len(tree.trees()) == 0

        tree['thing1'].make()
        tree['thing2'].make()
        tree['thing3'].make()

        assert len(tree.trees()) == 0

        tree['larry/'].make()
        tree['curly/'].make()

        assert len(tree.trees()) == 2

    def test_equal(self, tree):
        t1 = tree['a dir/']
        t2 = tree['another dir/']

        assert t1 != t2
        assert t1['.'] == t1

    def test_compare(self, tree):
        assert tree['bark/'] <= tree['dark/']

    def test_makedirs(self, tree):
        t1 = tree['a/ton of/stupid/bricks/'].makedirs()

        assert t1.exists

    def test_glob(self, tree):
        tm = tree['moe'].make()
        tl = tree['larry'].make()
        tc = tree['curly'].make()

        assert tl in tree.glob('*r*y')
        assert tc in tree.glob('*r*y')

    def test_glob_OSError(self, tree):
        with pytest.raises(OSError) as error:
            tree.glob('something.*')
        assert "Tree doesn't exist in the filesystem" in str(error)

    def test_walk(self, tree):
        # files
        tree['scipy'].make()
        tree['2016'].make()
        tree['sprint'].make()

        # directories with files
        t1 = tree['a_dir/has_no_name']
        t2 = tree['another_dir/bites_the_dust']
        t1.make()
        t2.make()

        roots_scandir = []
        dirs_scandir = []
        files_scandir = []
        all_roots = []
        all_trees = []
        all_leaves = []

        for root, dirs, files in os.walk(tree.abspath):
            # os.walk normally doesn't add slash to path, in order to
            # replicate the path given by tree.walk() we use os.path.join
            if root != tree.abspath:
                root = os.path.join(root, '')
            roots_scandir.append(root)
            for directory in dirs:
                # this is the abspath of the directory,
                # same reason as above for use of os.path.join
                dirs_scandir.append(os.path.join(root, directory, ''))
            for f in files:
                files_scandir.append(f)

        for root, trees, leaves in tree.walk():
            all_roots.append(root.abspath)
            for tree in trees:
                # this is the abspath of the directory
                all_trees.append(tree.abspath)
            for leaf in leaves:
                all_leaves.append(leaf.name)

        assert roots_scandir == all_roots
        assert dirs_scandir == all_trees
        assert files_scandir == all_leaves

    def test_walk_OSError(self, tree):
        with pytest.raises(OSError) as error:
            for v in tree.walk():  # need to use generator to trigger OSError?!
                assert v == 1
        assert "Tree doesn't exist in the filesystem" in str(error)

    def test_draw_OSError(self, tree):
        with pytest.raises(OSError) as error:
            tree.draw()
        assert "Tree doesn't exist in the filesystem" in str(error)


class TestLeaf(TestVeg):
    """Test Leaf-specific features.

    """
    cls = Leaf
    name = 'treeroot/a/fault/in/our/roots'

    @pytest.fixture
    def leaf(self, tmpdir):
        with tmpdir.as_cwd():
            l = Leaf(self.name)
            yield l
    veg = leaf

    def test_init(self, tmpdir):
        """
        Test that leaf works for:
            1. nonexistent file
            2. existing file

        Test that exception raised for:
            1. leaf initialized with existing directory

        """
        with tmpdir.as_cwd():

            # test nonexistent file
            t = Leaf('bark')
            assert not t.exists

            # test existent file
            t.make()
            assert t.exists

            t2 = Leaf('bark')
            assert t2.exists

            # test that init with directory raises ValueError

            # this should create a nonexistent Tree
            t3 = Tree('mark/').make()
            assert t3.exists

            with pytest.raises(ValueError):
                t4 = Leaf('mark')

    def test_exists(self, leaf):
        leaf.make()

        assert os.path.exists(leaf.abspath)
        assert leaf.exists is True

    def test_makedirs(self, leaf):
        pass


@pytest.fixture
def draw_tree(tmpdir):
    # set up a tree to draw with
    with tmpdir.as_cwd():
        t = Tree('here')
        t['file_zero'].make()
        t['.file_zero_hidden'].make()
        t['dir_one/'].make()
        t['dir_one/file_one'].make()
        t['dir_one/.file_one_hidden'].make()
        t['dir_one/dir_two/'].make()
        t['dir_one/dir_two/file_two'].make()
        t['dir_one/dir_two/.file_two_hidden'].make()

        yield t

DRAWREF_d0_T = """\
here/
 +-- .file_zero_hidden
 +-- file_zero
 +-- dir_one/
     +-- .file_one_hidden
     +-- file_one
     +-- dir_two/
         +-- .file_two_hidden
         +-- file_two
"""

DRAWREF_d0_F = """\
here/
 +-- file_zero
 +-- dir_one/
     +-- file_one
     +-- dir_two/
         +-- file_two
"""

DRAWREF_d1_T = """\
here/
 +-- .file_zero_hidden
 +-- file_zero
 +-- dir_one/
"""

DRAWREF_d1_F = """\
here/
 +-- file_zero
 +-- dir_one/
"""

DRAWREF_d2_T = """\
here/
 +-- .file_zero_hidden
 +-- file_zero
 +-- dir_one/
     +-- .file_one_hidden
     +-- file_one
     +-- dir_two/
"""

DRAWREF_d2_F = """\
here/
 +-- file_zero
 +-- dir_one/
     +-- file_one
     +-- dir_two/
"""


@pytest.mark.parametrize('depth,hidden,ref', (
    (None, True, DRAWREF_d0_T),
    (None, False, DRAWREF_d0_F),
    (1, True, DRAWREF_d1_T),
    (1, False, DRAWREF_d1_F),
    (2, True, DRAWREF_d2_T),
    (2, False, DRAWREF_d2_F),
    (3, True, DRAWREF_d0_T),  # 3 is max depth, so goes to full
    (3, False, DRAWREF_d0_F),
))
def test_tree_draw(draw_tree, depth, hidden, ref, capsys):
    draw_tree.draw(depth=depth, hidden=hidden)
    out, err = capsys.readouterr()
    assert out == ref



# -*-encoding: utf-8 -*-
import moderngl
import os

def source(uri, consts={}):
    """
    Reads and replaces constants (in all caps) in a text file.

    Parameters
    ----------
    uri : str
        URI of the file to read.
    consts : dict
        Dictionary containing the values to replace.

    Returns
    -------

    str
        File contents with replacements.

    """
    with open(uri, 'r') as fp:
        content = fp.read()

    # feed constant values
    for key, value in consts.items():
        content = content.replace(f"%%{key}%%".upper(), str(value))
    return content

def testGL():
    """
    Tries to initialize a compute shader.

    Returns
    -------

    bool
        Initialisation success.

    """
    f = source(os.path.dirname(__file__)+'/dummy.glsl', {"X":50})
    try:
        context = moderngl.create_standalone_context(require=430)
        context.compute_shader(f)
    except:
        return False
    else:
        return True
